import * as path from "path";
import express from "express";
import cookieParser from "cookie-parser";
import { fs, compatRequire, logger } from "@modern-js/utils";
import finalhandler from "finalhandler";
import { httpCallBack2HonoMid, sendResponse } from "@modern-js/server-core/base/node";
import { run } from "./context";
import registerRoutes from "./registerRoutes";
const findAppModule = async (apiDir) => {
  const exts = [
    ".ts",
    ".js"
  ];
  const paths = exts.map((ext) => path.resolve(apiDir, `app${ext}`));
  for (const filename of paths) {
    if (await fs.pathExists(filename)) {
      delete require.cache[filename];
      return [
        compatRequire(filename),
        require(filename)
      ];
    }
  }
  return [];
};
const initMiddlewares = (middleware, app) => {
  middleware.forEach((middlewareItem) => {
    const middlewareFunc = typeof middlewareItem === "string" ? compatRequire(middlewareItem) : middlewareItem;
    app.use(middlewareFunc);
  });
};
const useRun = (app) => {
  app.use((req, res, next) => {
    run({
      req,
      res
    }, next);
  });
};
const createApp = async ({ apiDir, middlewares, mode, apiHandlerInfos, render }) => {
  let app;
  if (mode === "framework") {
    const appModule = await findAppModule(apiDir);
    app = appModule[0];
    const hooks = appModule[1];
    if (!(app === null || app === void 0 ? void 0 : app.use)) {
      app = express();
    }
    initApp(app);
    if (middlewares && middlewares.length > 0) {
      initMiddlewares(middlewares, app);
    }
    useRun(app);
    registerRoutes(app, apiHandlerInfos);
    if (hooks) {
      const { afterLambdaRegisted } = hooks;
      if (afterLambdaRegisted) {
        afterLambdaRegisted(app);
      }
    }
  } else if (mode === "function") {
    app = express();
    initApp(app);
    if (middlewares && middlewares.length > 0) {
      initMiddlewares(middlewares, app);
    }
    useRun(app);
    registerRoutes(app, apiHandlerInfos);
  } else {
    throw new Error(`mode must be function or framework`);
  }
  if (render) {
    app.use(async (req, res, next) => {
      const response = await render(req.__honoRequest, {
        logger,
        nodeReq: req,
        templates: req.__templates,
        serverManifest: req.__serverManifest
      });
      if (response) {
        return sendResponse(response, res).then(next);
      }
      next();
    });
  }
  return app;
};
const initApp = (app) => {
  app.use(cookieParser());
  app.use(express.text());
  app.use(express.urlencoded({
    extended: true
  }));
  app.use(express.json());
  return app;
};
var plugin_default = () => {
  let app;
  let apiDir;
  let mode;
  let renderHtml;
  return {
    name: "@modern-js/plugin-express",
    pre: [
      "@modern-js/plugin-bff"
    ],
    post: [
      "@modern-js/plugin-server"
    ],
    setup: (api) => ({
      async onApiChange(changes) {
        const appContext = api.useAppContext();
        const middlewares = appContext.apiMiddlewares;
        const apiHandlerInfos = appContext.apiHandlerInfos;
        app = await createApp({
          apiDir,
          middlewares,
          mode,
          apiHandlerInfos,
          render: renderHtml
        });
        return changes;
      },
      async prepareApiServer({ pwd, render }) {
        var _userConfig_bff;
        const appContext = api.useAppContext();
        const apiHandlerInfos = appContext.apiHandlerInfos;
        const { apiDirectory } = appContext;
        const userConfig = api.useConfigContext();
        const middlewares = appContext.apiMiddlewares;
        mode = appContext.apiMode;
        renderHtml = ((_userConfig_bff = userConfig.bff) === null || _userConfig_bff === void 0 ? void 0 : _userConfig_bff.enableHandleWeb) && render ? render : void 0;
        apiDir = apiDirectory || path.join(pwd, "./api");
        app = await createApp({
          apiDir,
          middlewares,
          mode,
          apiHandlerInfos,
          render: renderHtml
        });
        const handler = (req, res) => new Promise((resolve, reject) => {
          const handler2 = (err) => {
            if (err) {
              return reject(err);
            }
            finalhandler(req, res, {})(null);
            return resolve();
          };
          res.on("finish", (err) => {
            if (err) {
              return reject(err);
            }
            return resolve();
          });
          return app(req, res, handler2);
        });
        return httpCallBack2HonoMid(handler);
      }
    })
  };
};
export {
  plugin_default as default
};
